/* light_source.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999-2001 DindinX <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include "giram.h"
#include "csgtree.h"
#include "snap.h"
#include "utils.h"
#include "light_source.h"

#include "giramwidgets.h"

#include "giramintl.h"

/*****************************************************************************
*  NewLightSourceOK
******************************************************************************/
void NewLightSourceOK(GtkWidget *Dialog)
{
  GtkObject         **Data;
  Vector              Position;
  Vector              Color;
  LightSourceStruct  *TmpLightSource;
  GSList             *tmp_list;
  ViewStruct         *TmpView;
  ViewStruct         *view_data;

  Data = gtk_object_get_data(GTK_OBJECT(Dialog), "WidgetData");
  view_data = gtk_object_get_data(Data[0], "ViewData");
  Position[0] = atof(gtk_entry_get_text(GTK_ENTRY(Data[1])));
  Position[1] = atof(gtk_entry_get_text(GTK_ENTRY(Data[2])));
  Position[2] = atof(gtk_entry_get_text(GTK_ENTRY(Data[3])));
  gtk_color_selection_get_color(GTK_COLOR_SELECTION(Data[4]), Color);

  TmpLightSource = g_new(LightSourceStruct, 1);
  Veq(TmpLightSource->Location, Position[0], Position[1], Position[2]);
  TmpLightSource->Color[0] = Color[0];
  TmpLightSource->Color[1] = Color[1];
  TmpLightSource->Color[2] = Color[2];
  TmpLightSource->Color[3] = 0.0;
  TmpLightSource->Color[4] = 0.0;
  TmpLightSource->Type = POINT_LIGHTSOURCE;
  Veq(TmpLightSource->PointAt, 0.0, 0.0, 1.0);
  TmpLightSource->Radius = 0.35;
  TmpLightSource->FallOff = 0.35;
  TmpLightSource->Tightness = 10.0;
  TmpLightSource->AreaLight = FALSE;
  Veq(TmpLightSource->Area1, 0.0, 0.0, 1.0);
  Veq(TmpLightSource->Area2, 0.0, 1.0, 0.0);
  TmpLightSource->AreaSize1 = 0.0;
  TmpLightSource->AreaSize2 = 0.0;
  TmpLightSource->Adaptive = 100.0;
  TmpLightSource->Jitter = FALSE;
  TmpLightSource->LooksLike = NULL;
  TmpLightSource->FadeDistance = 0.0;
  TmpLightSource->FadePower = 0.0;
  TmpLightSource->AtmosphericAttenuation = FALSE;
  TmpLightSource->Atmosphere = FALSE;
  view_data->frame->all_light_sources = g_slist_append(view_data->frame->all_light_sources,
                                                       TmpLightSource);
  for (tmp_list = view_data->frame->all_views ;
       tmp_list ;
       tmp_list = tmp_list->next)
  {
    TmpView = tmp_list->data;
    gtk_widget_queue_draw(TmpView->drawing_area);
  }
  RebuildCSGTree(NULL, view_data->frame);
  gtk_widget_destroy(Dialog);
}

/*****************************************************************************
*  ToolNewLightSource
******************************************************************************/
void ToolNewLightSource(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  GtkWidget   *Dialog, *ColorSel, *Table, *Label, *Entry;
  GtkObject  **Data;
  ViewStruct  *view_data;
  char         CoordText[20];
  double       x, y;

  view_data = get_current_view_data();

  Dialog = giram_dialog_new(_("New Light Source"), "giram_dialog",
                            NULL /* FIXME */,
                            "dialogs/new_light_source.html",
                            GTK_WIN_POS_MOUSE,
                            FALSE, FALSE, TRUE,

                            _("OK"), NewLightSourceOK,
                            NULL, 1, NULL, TRUE, TRUE,

                            NULL);
  Data = g_new(GtkObject*, 5);

  Data[0] = GTK_OBJECT(DrawingArea);

  gtk_object_set_data(GTK_OBJECT(Dialog), "WidgetData", Data);

  /* The Table for the position of the light */
  Table = gtk_table_new(3, 2, FALSE);
  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(Dialog)->vbox), Table);
  MouseToReal(bevent->x, bevent->y, DrawingArea, &x, &y);
  /* The X position */
  Label = gtk_label_new("Position X");
  gtk_table_attach_defaults(GTK_TABLE(Table), Label, 0,1, 0,1);
  Entry = gtk_entry_new();
  Data[1] = GTK_OBJECT(Entry);
  gtk_table_attach_defaults(GTK_TABLE(Table), Entry, 1,2, 0,1);
  switch (view_data->camera_style)
  {
    case ORTHO_XY_CAMERA:
      sprintf(CoordText, "%g", x);
      break;

    case ORTHO_XZ_CAMERA:
      sprintf(CoordText, "%g", x);
      break;

    case ORTHO_ZY_CAMERA:
      strcpy(CoordText, "0.0");
      break;
  }
  gtk_entry_set_text(GTK_ENTRY(Entry), CoordText);
  /* The Y position */
  Label = gtk_label_new("Position Y");
  gtk_table_attach_defaults(GTK_TABLE(Table), Label, 0,1, 1,2);
  Entry = gtk_entry_new();
  Data[2] = GTK_OBJECT(Entry);
  gtk_table_attach_defaults(GTK_TABLE(Table), Entry, 1,2, 1,2);
  switch (view_data->camera_style)
  {
    case ORTHO_XY_CAMERA:
      sprintf(CoordText, "%g", y);
      break;

    case ORTHO_XZ_CAMERA:
      strcpy(CoordText, "0.0");
      break;

    case ORTHO_ZY_CAMERA:
      sprintf(CoordText, "%g", y);
      break;
  }
  gtk_entry_set_text(GTK_ENTRY(Entry), CoordText);
  /* The Z position */
  Label = gtk_label_new("Position Z");
  gtk_table_attach_defaults(GTK_TABLE(Table), Label, 0,1, 2,3);
  Entry = gtk_entry_new();
  Data[3] = GTK_OBJECT(Entry);
  gtk_table_attach_defaults(GTK_TABLE(Table), Entry, 1,2, 2,3);
  switch (view_data->camera_style)
  {
    case ORTHO_XY_CAMERA:
      strcpy(CoordText, "0.0");
      break;

    case ORTHO_XZ_CAMERA:
      sprintf(CoordText, "%g", y);
      break;

    case ORTHO_ZY_CAMERA:
      sprintf(CoordText, "%g", x);
      break;
  }
  gtk_entry_set_text(GTK_ENTRY(Entry), CoordText);

  /* The Color Selector */
  ColorSel = gtk_color_selection_new();
  Data[4] = GTK_OBJECT(ColorSel);
  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(Dialog)->vbox), ColorSel);

  gtk_widget_show_all(Dialog);
}

