/* superellipsoid.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999-2000 DindinX <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "giram.h"
#include "trimesh.h"
#include "csgtree.h"
#include "object.h"
#include "utils.h"
#include "superellipsoid.h"

#include "giramintl.h"

static void giram_superellipsoid_build_triangle_mesh(ObjectStruct *object);
static gboolean giram_superellipsoid_inside(ObjectStruct *object, double x, double y, double z);
static gboolean giram_superellipsoid_find_intersection_segment(ObjectStruct *object,
                                                               Vector in_point, Vector out_point,
                                                               Vector inter_point, Vector inter_norm);

/*****************************************************************************
*  SuperEllipsoidMeshNormal FIXME
******************************************************************************/
static void SuperEllipsoidMeshNormal(Vector Result, ObjectStruct *SuperEllipsoid,
                                     Vector Inter)
{ /* FIXME */
  Result[0] = 0.0;
  Result[1] = 0.0;
  Result[2] = 0.0;
}

/*****************************************************************************
*  ComputeSuperEllipsoidTriangleMesh
******************************************************************************/
static void giram_superellipsoid_build_triangle_mesh(ObjectStruct *SuperEllipsoid)
{ /* XXX: clipped_by */
  Vector                P1, P2, P3, P4;
  Vector                N1, N2, N3, N4;
  int                   i, j;
  gint                  detail_level;
  SuperEllipsoidStruct *SSuperEllipsoid = (SuperEllipsoidStruct *)SuperEllipsoid;
  double                t1, s1, c1, t2, s2, c2;

  detail_level = SuperEllipsoid->detail_level;

  if (SuperEllipsoid->FirstTriangle)
    DestroyObjectTriangleMesh(SuperEllipsoid);
  /* First the North Pole */
  Veq(P1, 0.0, 0.0, 1.0);
  SuperEllipsoidMeshNormal(N1, SuperEllipsoid, P1);
  if (SuperEllipsoid->Trans)
  {
    MEvaluatePoint(P1, SuperEllipsoid->Trans, P1);
    MEvaluateVector(N1, SuperEllipsoid->Trans, N1);
  }
  for (i=0; i<detail_level; i++)
  { /* The "DetailLevel" triangles adjacents to the pole */
    t1 = M_PI / detail_level;
    if (sin(t1) >= 0.0) s1 =  pow( sin(t1), SSuperEllipsoid->N);
    else                s1 = -pow(-sin(t1), SSuperEllipsoid->N);
    if (cos(t1) >= 0.0) c1 =  pow( cos(t1), SSuperEllipsoid->N);
    else                c1 = -pow(-cos(t1), SSuperEllipsoid->N);
    t2 = i * 2.0 * M_PI / detail_level;
    if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
    else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
    if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
    else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
    Veq(P2, s1*c2, s1*s2, c1);
    SuperEllipsoidMeshNormal(N2, SuperEllipsoid, P2);
    if (SuperEllipsoid->Trans)
    {
      MEvaluatePoint(P2, SuperEllipsoid->Trans, P2);
      MEvaluateVector(N2, SuperEllipsoid->Trans, N2);
    }
    t2 = (i+1) * 2.0 * M_PI / detail_level;
    if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
    else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
    if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
    else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
    Veq(P3, s1*c2, s1*s2, c1);
    SuperEllipsoidMeshNormal(N3, SuperEllipsoid, P3);
    if (SuperEllipsoid->Trans)
    {
      MEvaluatePoint(P3, SuperEllipsoid->Trans, P3);
      MEvaluateVector(N3, SuperEllipsoid->Trans, N3);
    }
    AddTriangleToObjectMesh(SuperEllipsoid, P1,P2,P3, N1,N2,N3);
  }
  /* Then the Mid Zone */
  for (i=1 ; i<detail_level-1 ; i++)
  {
    for (j=0 ; j<detail_level ; j++)
    {
      t1 = i * M_PI / detail_level;
      if (sin(t1) >= 0.0) s1 =  pow( sin(t1), SSuperEllipsoid->N);
      else                s1 = -pow(-sin(t1), SSuperEllipsoid->N);
      if (cos(t1) >= 0.0) c1 =  pow( cos(t1), SSuperEllipsoid->N);
      else                c1 = -pow(-cos(t1), SSuperEllipsoid->N);
      t2 = j * 2.0 * M_PI / detail_level;
      if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
      else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
      if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
      else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
      Veq(P1, s1*c2, s1*s2, c1);
      SuperEllipsoidMeshNormal(N1, SuperEllipsoid, P1);
      if (SuperEllipsoid->Trans)
      {
        MEvaluatePoint(P1, SuperEllipsoid->Trans, P1);
        MEvaluateVector(N1, SuperEllipsoid->Trans, N1);
      }
      t1 = (i+1) * M_PI / detail_level;
      if (sin(t1) >= 0.0) s1 =  pow( sin(t1), SSuperEllipsoid->N);
      else                s1 = -pow(-sin(t1), SSuperEllipsoid->N);
      if (cos(t1) >= 0.0) c1 =  pow( cos(t1), SSuperEllipsoid->N);
      else                c1 = -pow(-cos(t1), SSuperEllipsoid->N);
      t2 = j * 2.0 * M_PI / detail_level;
      if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
      else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
      if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
      else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
      Veq(P2, s1*c2, s1*s2, c1);
      SuperEllipsoidMeshNormal(N2, SuperEllipsoid, P2);
      if (SuperEllipsoid->Trans)
      {
        MEvaluatePoint(P2, SuperEllipsoid->Trans, P2);
        MEvaluateVector(N2, SuperEllipsoid->Trans, N2);
      }
      t1 = i * M_PI / detail_level;
      if (sin(t1) >= 0.0) s1 =  pow( sin(t1), SSuperEllipsoid->N);
      else                s1 = -pow(-sin(t1), SSuperEllipsoid->N);
      if (cos(t1) >= 0.0) c1 =  pow( cos(t1), SSuperEllipsoid->N);
      else                c1 = -pow(-cos(t1), SSuperEllipsoid->N);
      t2 = (j+1) * 2.0 * M_PI / detail_level;
      if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
      else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
      if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
      else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
      Veq(P3, s1*c2, s1*s2, c1);
      SuperEllipsoidMeshNormal(N3, SuperEllipsoid, P3);
      if (SuperEllipsoid->Trans)
      {
        MEvaluatePoint(P3, SuperEllipsoid->Trans, P3);
        MEvaluateVector(N3, SuperEllipsoid->Trans, N3);
      }
      t1 = (i+1) * M_PI / detail_level;
      if (sin(t1) >= 0.0) s1 =  pow( sin(t1), SSuperEllipsoid->N);
      else                s1 = -pow(-sin(t1), SSuperEllipsoid->N);
      if (cos(t1) >= 0.0) c1 =  pow( cos(t1), SSuperEllipsoid->N);
      else                c1 = -pow(-cos(t1), SSuperEllipsoid->N);
      t2 = (j+1) * 2.0 * M_PI / detail_level;
      if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
      else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
      if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
      else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
      Veq(P4, s1*c2, s1*s2, c1);
      SuperEllipsoidMeshNormal(N4, SuperEllipsoid, P4);
      if (SuperEllipsoid->Trans)
      {
        MEvaluatePoint(P4, SuperEllipsoid->Trans, P4);
        MEvaluateVector(N4, SuperEllipsoid->Trans, N4);
      }
      AddTriangleToObjectMesh(SuperEllipsoid, P1,P2,P4, N1,N2,N4);
      AddTriangleToObjectMesh(SuperEllipsoid, P1,P3,P4, N1,N3,N4);
    }
  }
  /* And the South Pole */
  Veq(P1, 0.0, 0.0, -1.0);
  SuperEllipsoidMeshNormal(N1, SuperEllipsoid, P1);
  if (SuperEllipsoid->Trans)
  {
    MEvaluatePoint(P1, SuperEllipsoid->Trans, P1);
    MEvaluateVector(N1, SuperEllipsoid->Trans, N1);
  }
  for (i=0 ; i<detail_level ; i++)
  { /* The "DetailLevel" triangles adjacents to the south pole */
    t1 = M_PI - M_PI / detail_level;
    if (sin(t1) >= 0.0) s1 =  pow( sin(t1), SSuperEllipsoid->N);
    else                s1 = -pow(-sin(t1), SSuperEllipsoid->N);
    if (cos(t1) >= 0.0) c1 =  pow( cos(t1), SSuperEllipsoid->N);
    else                c1 = -pow(-cos(t1), SSuperEllipsoid->N);
    t2 = i * 2.0 * M_PI / detail_level;
    if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
    else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
    if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
    else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
    Veq(P2, s1*c2, s1*s2, c1);
    SuperEllipsoidMeshNormal(N2, SuperEllipsoid, P2);
    if (SuperEllipsoid->Trans)
    {
      MEvaluatePoint(P2, SuperEllipsoid->Trans, P2);
      MEvaluateVector(N2, SuperEllipsoid->Trans, N2);
    }
    t2 = (i+1) * 2.0 * M_PI / detail_level;
    if (sin(t2) >= 0.0) s2 =  pow( sin(t2), SSuperEllipsoid->E);
    else                s2 = -pow(-sin(t2), SSuperEllipsoid->E);
    if (cos(t2) >= 0.0) c2 =  pow( cos(t2), SSuperEllipsoid->E);
    else                c2 = -pow(-cos(t2), SSuperEllipsoid->E);
    Veq(P3, s1*c2, s1*s2, c1);
    SuperEllipsoidMeshNormal(N3, SuperEllipsoid, P3);
    if (SuperEllipsoid->Trans)
    {
      MEvaluatePoint(P3, SuperEllipsoid->Trans, P3);
      MEvaluateVector(N3, SuperEllipsoid->Trans, N3);
    }
    AddTriangleToObjectMesh(SuperEllipsoid, P1,P2,P3, N1,N2,N3);
  }
}

/*****************************************************************************
*  giram_superellipsoid_inside
******************************************************************************/
static gboolean giram_superellipsoid_inside(ObjectStruct *object, double x, double y, double z)
{
  Vector Vect;

  Veq(Vect, x, y, z);
  if (object->Trans)
  {
    MInverseEvaluatePoint(Vect, object->Trans, Vect);
  }
  return FALSE; /* FIXME */
}

/*****************************************************************************
*  giram_superellipsoid_find_intersection_segment
******************************************************************************/
static gboolean giram_superellipsoid_find_intersection_segment(ObjectStruct *object,
                                                               Vector in_point, Vector out_point,
                                                               Vector inter_point, Vector inter_norm)
{
  return FALSE; /* FIXME */
}

/****************************************************************************
*  giram_superellipsoid_dialog_area_expose
*****************************************************************************/
static gboolean giram_superellipsoid_dialog_area_expose(GtkWidget *area,
                                                        GdkEventExpose *ev,
                                                        GtkWidget *dialog)
{
  GtkAdjustment   *adj;
  GtkWidget       *spin;
  gdouble          ns, ew;
  Vector           P1, P2, P3, P4, TP1, TP2, TP3, TP4;
  int              i,j;
  double           t1, s1, c1, t2, s2, c2;
  Vector           rotate;
  TransformStruct  transform;
  GdkPixmap        *pixmap;
  gint              detail_level;

  pixmap = gdk_pixmap_new(area->window,
                          area->allocation.width,
                          area->allocation.height,
                          -1);
  gdk_draw_rectangle(pixmap,
                     area->style->bg_gc[GTK_STATE_NORMAL],
                     TRUE, 0, 0,
                     area->allocation.width,
                     area->allocation.height);

  adj  = gtk_object_get_data(GTK_OBJECT(dialog), "east-west");
  ew   = adj->value;
  adj  = gtk_object_get_data(GTK_OBJECT(dialog), "north-south");
  ns   = adj->value;

  adj       = gtk_object_get_data(GTK_OBJECT(dialog), "rotate-x");
  rotate[0] = adj->value;
  adj       = gtk_object_get_data(GTK_OBJECT(dialog), "rotate-y");
  rotate[1] = adj->value;
  rotate[2] = 0.0;
  MIdentity(transform.Direct);
  MIdentity(transform.Inverse);
  ComputeRotateTrans(&transform, rotate);

  spin = gtk_object_get_data(GTK_OBJECT(dialog), "detail");
  detail_level = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));

  /* First the North Pole */
  Veq(P1, 0.0, 0.0, 1.0);
  for (i=0; i<detail_level; i++)
  { /* The triangles which are adjacents to the pole */
    t1 = M_PI / detail_level;
    if (sin(t1) >= 0.0) s1 =  pow( sin(t1), ns);
    else                s1 = -pow(-sin(t1), ns);
    if (cos(t1) >= 0.0) c1 =  pow( cos(t1), ns);
    else                c1 = -pow(-cos(t1), ns);
    t2 = i * 2.0 * M_PI / detail_level;
    if (sin(t2) >= 0.0) s2 =  pow( sin(t2), ew);
    else                s2 = -pow(-sin(t2), ew);
    if (cos(t2) >= 0.0) c2 =  pow( cos(t2), ew);
    else                c2 = -pow(-cos(t2), ew);
    Veq(P2, s1*c2, s1*s2, c1);
    t2 = (i+1) * 2.0 * M_PI / detail_level;
    if (sin(t2) >= 0.0) s2 =  pow( sin(t2), ew);
    else                s2 = -pow(-sin(t2), ew);
    if (cos(t2) >= 0.0) c2 =  pow( cos(t2), ew);
    else                c2 = -pow(-cos(t2), ew);
    Veq(P3, s1*c2, s1*s2, c1);
    MEvaluatePoint(TP1, &transform, P1);
    MEvaluatePoint(TP2, &transform, P2);
    MEvaluatePoint(TP3, &transform, P3);
    gdk_draw_line(pixmap, area->style->black_gc,
                  area->allocation.width /2+TP1[0]*100,
                  area->allocation.height/2-TP1[1]*100,
                  area->allocation.width /2+TP2[0]*100,
                  area->allocation.height/2-TP2[1]*100);
    gdk_draw_line(pixmap, area->style->black_gc,
                  area->allocation.width /2+TP1[0]*100,
                  area->allocation.height/2-TP1[1]*100,
                  area->allocation.width /2+TP3[0]*100,
                  area->allocation.height/2-TP3[1]*100);
    gdk_draw_line(pixmap, area->style->black_gc,
                  area->allocation.width /2+TP3[0]*100,
                  area->allocation.height/2-TP3[1]*100,
                  area->allocation.width /2+TP2[0]*100,
                  area->allocation.height/2-TP2[1]*100);
  }
  /* Then the Mid Zone */
  for (i=1 ; i<detail_level-1 ; i++)
  {
    for (j=0 ; j<detail_level ; j++)
    {
      t1 = i * M_PI / detail_level;
      if (sin(t1) >= 0.0) s1 =  pow( sin(t1), ns);
      else                s1 = -pow(-sin(t1), ns);
      if (cos(t1) >= 0.0) c1 =  pow( cos(t1), ns);
      else                c1 = -pow(-cos(t1), ns);
      t2 = j * 2.0 * M_PI / detail_level;
      if (sin(t2) >= 0.0) s2 =  pow( sin(t2), ew);
      else                s2 = -pow(-sin(t2), ew);
      if (cos(t2) >= 0.0) c2 =  pow( cos(t2), ew);
      else                c2 = -pow(-cos(t2), ew);
      Veq(P1, s1*c2, s1*s2, c1);
      t1 = (i+1) * M_PI / detail_level;
      if (sin(t1) >= 0.0) s1 =  pow( sin(t1), ns);
      else                s1 = -pow(-sin(t1), ns);
      if (cos(t1) >= 0.0) c1 =  pow( cos(t1), ns);
      else                c1 = -pow(-cos(t1), ns);
      t2 = j * 2.0 * M_PI / detail_level;
      if (sin(t2) >= 0.0) s2 =  pow( sin(t2), ew);
      else                s2 = -pow(-sin(t2), ew);
      if (cos(t2) >= 0.0) c2 =  pow( cos(t2), ew);
      else                c2 = -pow(-cos(t2), ew);
      Veq(P2, s1*c2, s1*s2, c1);
      t1 = i * M_PI / detail_level;
      if (sin(t1) >= 0.0) s1 =  pow( sin(t1), ns);
      else                s1 = -pow(-sin(t1), ns);
      if (cos(t1) >= 0.0) c1 =  pow( cos(t1), ns);
      else                c1 = -pow(-cos(t1), ns);
      t2 = (j+1) * 2.0 * M_PI / detail_level;
      if (sin(t2) >= 0.0) s2 =  pow( sin(t2), ew);
      else                s2 = -pow(-sin(t2), ew);
      if (cos(t2) >= 0.0) c2 =  pow( cos(t2), ew);
      else                c2 = -pow(-cos(t2), ew);
      Veq(P3, s1*c2, s1*s2, c1);
      t1 = (i+1) * M_PI / detail_level;
      if (sin(t1) >= 0.0) s1 =  pow( sin(t1), ns);
      else                s1 = -pow(-sin(t1), ns);
      if (cos(t1) >= 0.0) c1 =  pow( cos(t1), ns);
      else                c1 = -pow(-cos(t1), ns);
      t2 = (j+1) * 2.0 * M_PI / detail_level;
      if (sin(t2) >= 0.0) s2 =  pow( sin(t2), ew);
      else                s2 = -pow(-sin(t2), ew);
      if (cos(t2) >= 0.0) c2 =  pow( cos(t2), ew);
      else                c2 = -pow(-cos(t2), ew);
      Veq(P4, s1*c2, s1*s2, c1);
      MEvaluatePoint(TP1, &transform, P1);
      MEvaluatePoint(TP2, &transform, P2);
      MEvaluatePoint(TP3, &transform, P3);
      MEvaluatePoint(TP4, &transform, P4);
      gdk_draw_line(pixmap, area->style->black_gc,
                    area->allocation.width /2+TP1[0]*100,
                    area->allocation.height/2-TP1[1]*100,
                    area->allocation.width /2+TP2[0]*100,
                    area->allocation.height/2-TP2[1]*100);
      gdk_draw_line(pixmap, area->style->black_gc,
                    area->allocation.width /2+TP2[0]*100,
                    area->allocation.height/2-TP2[1]*100,
                    area->allocation.width /2+TP4[0]*100,
                    area->allocation.height/2-TP4[1]*100);
      gdk_draw_line(pixmap, area->style->black_gc,
                    area->allocation.width /2+TP4[0]*100,
                    area->allocation.height/2-TP4[1]*100,
                    area->allocation.width /2+TP3[0]*100,
                    area->allocation.height/2-TP3[1]*100);
      gdk_draw_line(pixmap, area->style->black_gc,
                    area->allocation.width /2+TP3[0]*100,
                    area->allocation.height/2-TP3[1]*100,
                    area->allocation.width /2+TP1[0]*100,
                    area->allocation.height/2-TP1[1]*100);
    }
  }
  /* And the South Pole */
  Veq(P1, 0.0, 0.0, -1.0);
  for (i=0 ; i<detail_level ; i++)
  { /* The triangles which are adjacents to the south pole */
    t1 = M_PI - M_PI / detail_level;
    if (sin(t1) >= 0.0) s1 =  pow( sin(t1), ns);
    else                s1 = -pow(-sin(t1), ns);
    if (cos(t1) >= 0.0) c1 =  pow( cos(t1), ns);
    else                c1 = -pow(-cos(t1), ns);
    t2 = i * 2.0 * M_PI / detail_level;
    if (sin(t2) >= 0.0) s2 =  pow( sin(t2), ew);
    else                s2 = -pow(-sin(t2), ew);
    if (cos(t2) >= 0.0) c2 =  pow( cos(t2), ew);
    else                c2 = -pow(-cos(t2), ew);
    Veq(P2, s1*c2, s1*s2, c1);
    t2 = (i+1) * 2.0 * M_PI / detail_level;
    if (sin(t2) >= 0.0) s2 =  pow( sin(t2), ew);
    else                s2 = -pow(-sin(t2), ew);
    if (cos(t2) >= 0.0) c2 =  pow( cos(t2), ew);
    else                c2 = -pow(-cos(t2), ew);
    Veq(P3, s1*c2, s1*s2, c1);
    MEvaluatePoint(TP1, &transform, P1);
    MEvaluatePoint(TP2, &transform, P2);
    MEvaluatePoint(TP3, &transform, P3);
    gdk_draw_line(pixmap, area->style->black_gc,
                  area->allocation.width /2+TP1[0]*100,
                  area->allocation.height/2-TP1[1]*100,
                  area->allocation.width /2+TP2[0]*100,
                  area->allocation.height/2-TP2[1]*100);
    gdk_draw_line(pixmap, area->style->black_gc,
                  area->allocation.width /2+TP1[0]*100,
                  area->allocation.height/2-TP1[1]*100,
                  area->allocation.width /2+TP3[0]*100,
                  area->allocation.height/2-TP3[1]*100);
    gdk_draw_line(pixmap, area->style->black_gc,
                  area->allocation.width /2+TP3[0]*100,
                  area->allocation.height/2-TP3[1]*100,
                  area->allocation.width /2+TP2[0]*100,
                  area->allocation.height/2-TP2[1]*100);
  }
  gdk_window_set_back_pixmap(area->window, pixmap, FALSE);
  gdk_window_clear(area->window);
  gdk_pixmap_unref(pixmap);
  return TRUE;
}

/*****************************************************************************
*  NewSuperEllipsoidOK
******************************************************************************/
static void NewSuperEllipsoidOK(GtkWidget *dialog)
{
  GtkWidget     *area, *spin;
  ViewStruct    *view_data;
  ObjectStruct  *SuperEllipsoid;
  GSList        *tmp_list;
  ViewStruct    *TmpView;
  GtkAdjustment *adj;
  gdouble        ns, ew;
  Vector         rotate, translate;
  gdouble       *get_trans;

  area = gtk_object_get_data(GTK_OBJECT(dialog), "area");
  view_data = gtk_object_get_data(GTK_OBJECT(area), "ViewData");
  adj = gtk_object_get_data(GTK_OBJECT(dialog), "east-west");
  ew = adj->value;
  adj = gtk_object_get_data(GTK_OBJECT(dialog), "north-south");
  ns = adj->value;
  adj       = gtk_object_get_data(GTK_OBJECT(dialog), "rotate-x");
  rotate[0] = adj->value;
  adj       = gtk_object_get_data(GTK_OBJECT(dialog), "rotate-y");
  rotate[1] = adj->value;
  rotate[2] = 0.0;
  SuperEllipsoid = giram_superellipsoid_new(ns, ew);

  spin = gtk_object_get_data(GTK_OBJECT(dialog), "detail");
  SuperEllipsoid->detail_level = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
  view_data->frame->all_objects = g_slist_append(view_data->frame->all_objects,
                                                 SuperEllipsoid);
  giram_object_rotate(SuperEllipsoid, rotate);

  get_trans = gtk_object_get_data(GTK_OBJECT(dialog), "translate");
  Vcopy(translate, get_trans);
  giram_object_translate(SuperEllipsoid, translate);
  giram_object_build_triangle_mesh(SuperEllipsoid);
  for (tmp_list = view_data->frame->all_views ;
       tmp_list ;
       tmp_list = tmp_list->next)
  {
    TmpView = tmp_list->data;
    gtk_widget_queue_draw(TmpView->drawing_area);
  }
  RebuildCSGTree(NULL, view_data->frame);
  gtk_widget_destroy(dialog);
}

/*****************************************************************************
*  ToolNewSuperEllipsoid
******************************************************************************/
void ToolNewSuperEllipsoid(GtkWidget *DrawingArea, GdkEventButton *bevent)
{
  GtkWidget     *dialog, *button, *table, *label;
  GtkWidget     *canvas;
  GtkAdjustment *adj;
  GtkWidget     *scale;
  GtkWidget     *hbox, *spin;
  gdouble       *translate;
  gdouble        Xorg, Yorg;
  ViewStruct    *view_data;

  /* remember the translation vector */
  translate = g_new(gdouble, 3);
  MouseToReal(bevent->x, bevent->y, DrawingArea, &Xorg, &Yorg);
  view_data = gtk_object_get_data(GTK_OBJECT(DrawingArea), "ViewData");
  switch (view_data->camera_style)
  {
    case ORTHO_XY_CAMERA:
      Veq(translate, Xorg, Yorg, 0.0);
      break;

    case ORTHO_XZ_CAMERA:
      Veq(translate, Xorg, 0.0, Yorg);
      break;

    case ORTHO_ZY_CAMERA:
      Veq(translate, 0.0, Yorg, Xorg);
      break;
  }

  dialog = giram_dialog_new(_("New SuperEllipsoid"), "giram_dialog",
                            NULL /* FIXME */,
                            "dialogs/new_superellipsoid.html",
                            GTK_WIN_POS_MOUSE, FALSE, TRUE, FALSE,
                            _("OK"), NewSuperEllipsoidOK,
                            NULL, 1, NULL, TRUE, FALSE,
                            _("Cancel"), gtk_widget_destroy,
                            NULL, 1, NULL, FALSE, TRUE,
                            NULL);
  gtk_object_set_data(GTK_OBJECT(dialog), "translate", translate);
  gtk_object_set_data(GTK_OBJECT(dialog), "area", DrawingArea);

  /* The canvas, for user feedback, sourrounded by to slider to rotate the SE. */
  table = gtk_table_new(2, 2, FALSE);
  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), table);

  canvas = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(canvas), 200, 200);
  gtk_table_attach_defaults(GTK_TABLE(table), canvas, 0,1, 0,1);
  gtk_signal_connect(GTK_OBJECT(canvas), "expose_event",
                     (GtkSignalFunc)giram_superellipsoid_dialog_area_expose,
                     dialog);
  /* The vertical slider for a rotation around the X axis */
  adj = (GtkAdjustment *)gtk_adjustment_new(0.0, -180.0, 180.0, 1.0, 5.0, 5.0);
  gtk_signal_connect_object(GTK_OBJECT(adj), "value_changed",
                            (GtkSignalFunc)gtk_widget_queue_draw,
                            GTK_OBJECT(canvas));
  scale = gtk_vscale_new(adj);
  gtk_table_attach(GTK_TABLE(table), scale, 1,2, 0,1,
                   GTK_FILL, GTK_FILL, 0,0);
  gtk_object_set_data(GTK_OBJECT(dialog), "rotate-x", adj);

  /* The horizontal slider for a rotation around the Y axis */
  adj = (GtkAdjustment *)gtk_adjustment_new(0.0, -180.0, 180.0, 1.0, 5.0, 5.0);
  gtk_signal_connect_object(GTK_OBJECT(adj), "value_changed",
                            (GtkSignalFunc)gtk_widget_queue_draw,
                            GTK_OBJECT(canvas));
  scale = gtk_hscale_new(adj);
  gtk_table_attach(GTK_TABLE(table), scale, 0,1, 1,2,
                   GTK_FILL, GTK_FILL, 0,0);
  gtk_object_set_data(GTK_OBJECT(dialog), "rotate-y", adj);

  /* The East-West Parameter */
  table = gtk_table_new(2, 2, FALSE);
  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), table);

  label = gtk_label_new(_("East-West Parameter: "));
  gtk_table_attach(GTK_TABLE(table), label, 0,1, 0,1,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);

  adj = (GtkAdjustment *)gtk_adjustment_new(1.0, 0.1, 5.0, 0.1, 0.1, 1.0);
  gtk_signal_connect_object(GTK_OBJECT(adj), "value_changed",
                            (GtkSignalFunc)gtk_widget_queue_draw,
                            GTK_OBJECT(canvas));
  scale = gtk_hscale_new(adj);
  gtk_widget_set_usize(scale, 150, 30);
  gtk_range_set_update_policy(GTK_RANGE(scale), GTK_UPDATE_CONTINUOUS);
  gtk_scale_set_draw_value(GTK_SCALE(scale), TRUE);
  gtk_table_attach(GTK_TABLE(table), scale, 1,2, 0,1,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_object_set_data(GTK_OBJECT(dialog), "east-west", adj);

  /* The North-South Parameter */
  label = gtk_label_new(_("North-South Parameter: "));
  gtk_table_attach(GTK_TABLE(table), label, 0,1, 1,2,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);

  adj = (GtkAdjustment*)gtk_adjustment_new(1.0, 0.1, 5.0, 0.1, 0.1, 1.0);
  gtk_signal_connect_object(GTK_OBJECT(adj), "value_changed",
                            (GtkSignalFunc)gtk_widget_queue_draw,
                            GTK_OBJECT(canvas));
  scale = gtk_hscale_new(adj);
  gtk_widget_set_usize(scale, 150, 30);
  gtk_range_set_update_policy(GTK_RANGE(scale), GTK_UPDATE_CONTINUOUS);
  gtk_scale_set_draw_value(GTK_SCALE(scale), TRUE);
  gtk_table_attach(GTK_TABLE(table), scale, 1,2, 1,2,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                   GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
  gtk_object_set_data(GTK_OBJECT(dialog), "north-south", adj);

  /* The detail level spin button */
  hbox = gtk_hbox_new(FALSE, 4);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 0);
  label = gtk_label_new(_("Detail Level:"));
  gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
  adj = (GtkAdjustment*)gtk_adjustment_new(16, 4, 128, 1, 4, 4);
  gtk_signal_connect_object(GTK_OBJECT(adj), "value_changed",
                            (GtkSignalFunc)gtk_widget_queue_draw,
                            GTK_OBJECT(canvas));
  spin = gtk_spin_button_new(adj, 1.0, 0);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), spin);
  gtk_object_set_data(GTK_OBJECT(dialog), "detail", spin);

  gtk_widget_show_all(dialog);
}

/*****************************************************************************
*  giram_superellipsoid_new
******************************************************************************/
ObjectStruct *giram_superellipsoid_new(double N, double E)
{
  ObjectStruct *SuperEllipsoid;
  SuperEllipsoidStruct *SSuperEllipsoid;
  static GiramObjectClass *superellipsoid_klass = NULL;

  if (superellipsoid_klass == NULL)
  {
    superellipsoid_klass = giram_object_class_new();

    superellipsoid_klass->build_triangle_mesh       = giram_superellipsoid_build_triangle_mesh;
    superellipsoid_klass->inside                    = giram_superellipsoid_inside;
    superellipsoid_klass->find_intersection_segment = giram_superellipsoid_find_intersection_segment;
  }
  SSuperEllipsoid = g_new(SuperEllipsoidStruct, 1);
  SuperEllipsoid = (ObjectStruct *)SSuperEllipsoid;
  InitObject(SuperEllipsoid);
  SuperEllipsoid->Type = SUPERELLIPSOID_OBJECT;
  SuperEllipsoid->klass = superellipsoid_klass;
  SuperEllipsoid->detail_level = 16;
  SSuperEllipsoid->E = E;
  SSuperEllipsoid->N = N;
  return SuperEllipsoid;
}

/*****************************************************************************
*  PDBCreateSuperEllipsoid
******************************************************************************/
int PDBCreateSuperEllipsoid(int *Id, double *N, double *E)
{
  ObjectStruct         *SuperEllipsoid;
  SuperEllipsoidStruct *SSuperEllipsoid;
  FrameStruct          *LocalFrame;
  GSList               *tmp_list;
  ViewStruct           *TmpView;

  SuperEllipsoid = giram_superellipsoid_new(*N, *E);

  LocalFrame = NULL;
  for (tmp_list = all_frames ; tmp_list ; tmp_list = g_slist_next(tmp_list) )
  {
    LocalFrame = tmp_list->data;
    if (LocalFrame->Id == *Id)
      break;
  }
  if (LocalFrame == NULL)
  {
    g_message("PDBCreateSuperEllipsoid() called with an unknown Id (%d)\n", *Id);
    return 0;
  }
  LocalFrame->all_objects = g_slist_append(LocalFrame->all_objects, SuperEllipsoid);
  giram_object_build_triangle_mesh(SuperEllipsoid);
  for (tmp_list = LocalFrame->all_views ;
       tmp_list ;
       tmp_list = tmp_list->next)
  {
    TmpView = tmp_list->data;
    gtk_widget_queue_draw(TmpView->drawing_area);
  }
  RebuildCSGTree(NULL, LocalFrame);
  return SuperEllipsoid->Id;
}

