/* copy.c
 * Giram - A GPLed Modelling Program.
 * Copyright (C) 1999-2000 DindinX <David.Odin@bigfoot.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include "giram.h"
#include "object.h"
#include "copy.h"

/*****************************************************************************
*  CopyObject
******************************************************************************/
ObjectStruct *CopyObject(ObjectStruct *OrgObject)
{
  ObjectStruct *LocalObject;

  if (OrgObject)
  {
    switch (OrgObject->Type)
    {
      case BOX_OBJECT:
        LocalObject = g_new(BoxStruct, 1);
        {
          BoxStruct *BBox = (BoxStruct *)LocalObject;
          BoxStruct *OrgBox = (BoxStruct *)OrgObject;

          Vcopy(BBox->MinCorner, OrgBox->MinCorner);
          Vcopy(BBox->MaxCorner, OrgBox->MaxCorner);
        }
        break;
      case CONE_OBJECT:
        LocalObject = g_new(ConeStruct, 1);
        {
          ConeStruct *CCone = (ConeStruct *)LocalObject;
          ConeStruct *OrgCone = (ConeStruct *)OrgObject;

          CCone->Open = OrgCone->Open;
          CCone->Radius1 = OrgCone->Radius1;
          CCone->Radius2 = OrgCone->Radius2;
        }
        break;
      case CSG_OBJECT:
        LocalObject = g_new(CSGStruct, 1);
        {
          CSGStruct *CCSG = (CSGStruct *)LocalObject;
          CSGStruct *OrgCSG = (CSGStruct *)OrgObject;
          ObjectStruct *Object1, *Object2;

          CCSG->Type = OrgCSG->Type;
/*          CCSG->FirstObject = CopyObject(OrgCSG->FirstObject);
          Object1 = CCSG->FirstObject;
          Object2 = OrgCSG->FirstObject;
          while (Object2)
          {
            Object1->Next = CopyObject(Object2->Next);
            Object1 = Object1->Next;
            Object2 = Object2->Next;
          }*/
        }
        break;
      case CYLINDER_OBJECT:
        LocalObject = g_new(CylinderStruct, 1);
        {
          CylinderStruct *CCylinder = (CylinderStruct *)LocalObject;
          CylinderStruct *OrgCylinder = (CylinderStruct *)OrgObject;

          CCylinder->Open = OrgCylinder->Open;
        }
        break;
      case DISC_OBJECT:
        LocalObject = g_new(DiscStruct, 1);
        {
          DiscStruct *DDisc = (DiscStruct *)LocalObject;
          DiscStruct *OrgDisc = (DiscStruct *)OrgObject;

          DDisc->Radius = OrgDisc->Radius;
          DDisc->HoleRadius = DDisc->HoleRadius;
        }
        break;
      case MESH_OBJECT:
        LocalObject = g_new(MeshStruct, 1);
        {
          MeshStruct *MMesh = (MeshStruct *)LocalObject;
          MeshStruct *OrgMesh = (MeshStruct *)OrgObject;
          TriangleListStruct *TmpTri, *NewTri;

          MMesh->FirstTriangle = NULL;
          for (TmpTri = OrgMesh->FirstTriangle ; TmpTri ; TmpTri=TmpTri->Next)
          {
            NewTri = g_new(TriangleListStruct, 1);
            NewTri->Next = MMesh->FirstTriangle;
            MMesh->FirstTriangle = NewTri;
            Vcopy(NewTri->P1, TmpTri->P1); Vcopy(NewTri->N1, TmpTri->N1);
            Vcopy(NewTri->P2, TmpTri->P2); Vcopy(NewTri->N2, TmpTri->N2);
            Vcopy(NewTri->P3, TmpTri->P3); Vcopy(NewTri->N3, TmpTri->N3);
          }
        }
        break;
      case PLANE_OBJECT:
        LocalObject = g_new(PlaneStruct, 1);
        {
          PlaneStruct *PPlane = (PlaneStruct *)LocalObject;
          PlaneStruct *OrgPlane = (PlaneStruct *)OrgObject;

          Vcopy(PPlane->Normal, OrgPlane->Normal);
          PPlane->Distance = OrgPlane->Distance;
          PPlane->NbSquare = OrgPlane->NbSquare;
          PPlane->SquareSize = OrgPlane->SquareSize;
        }
        break;
      case SOR_OBJECT:
        LocalObject = g_new(SorStruct, 1);
        {
          SorStruct *SSor = (SorStruct *)LocalObject;
          SorStruct *OrgSor = (SorStruct *)OrgObject;
          guint i;

          SSor->NumberOfPoints = OrgSor->NumberOfPoints;
          SSor->Point = g_new(Vector, SSor->NumberOfPoints);
          for (i=0 ; i<SSor->NumberOfPoints ; i++)
          {
            SSor->Point[i][0] = OrgSor->Point[i][0];
            SSor->Point[i][1] = OrgSor->Point[i][1];
          }
        }
        break;
      case SPHERE_OBJECT:
        LocalObject = g_new(SphereStruct, 1);
        {
          SphereStruct *SSphere = (SphereStruct *)LocalObject;
          SphereStruct *OrgSphere = (SphereStruct *)OrgObject;

          Vcopy(SSphere->Center, OrgSphere->Center);
          SSphere->Radius = OrgSphere->Radius;
        }
        break;
      case SUPERELLIPSOID_OBJECT:
        LocalObject = g_new(SuperEllipsoidStruct, 1);
        {
          SuperEllipsoidStruct *SSuperEllipsoid = (SuperEllipsoidStruct *)LocalObject;
          SuperEllipsoidStruct *OrgSuperEllipsoid = (SuperEllipsoidStruct *)OrgObject;

          SSuperEllipsoid->E = OrgSuperEllipsoid->E;
          SSuperEllipsoid->N = OrgSuperEllipsoid->N;
        }
        break;
      case TORUS_OBJECT:
        LocalObject = g_new(TorusStruct, 1);
        {
          TorusStruct *TTorus = (TorusStruct *)LocalObject;
          TorusStruct *OrgTorus = (TorusStruct *)OrgObject;

          TTorus->Major = OrgTorus->Major;
          TTorus->Minor = OrgTorus->Minor;
        }
        break;
      case TRIANGLE_OBJECT:
        LocalObject = g_new(TriangleStruct, 1);
        {
          TriangleStruct *TTriangle = (TriangleStruct *)LocalObject;
          TriangleStruct *OrgTriangle = (TriangleStruct *)OrgObject;

          Vcopy(TTriangle->P1, OrgTriangle->P1);
          Vcopy(TTriangle->P2, OrgTriangle->P2);
          Vcopy(TTriangle->P3, OrgTriangle->P3);
          Vcopy(TTriangle->N1, OrgTriangle->N1);
          Vcopy(TTriangle->N2, OrgTriangle->N2);
          Vcopy(TTriangle->N3, OrgTriangle->N3);
        }
        break;
      default:
        LocalObject = NULL;
        printf("Severe bug in CopyObject\n");
        exit(1);
    }
    LocalObject->Type = OrgObject->Type;
    LocalObject->FirstTriangle = NULL;
    LocalObject->Trans = CopyTransform(OrgObject->Trans);
    LocalObject->Texture = CopyTexture(OrgObject->Texture);
    LocalObject->Inverse = OrgObject->Inverse;
    LocalObject->NoShadow = OrgObject->NoShadow;
    LocalObject->Hollow = OrgObject->Hollow;
    LocalObject->selected = OrgObject->selected;
    LocalObject->visible = OrgObject->visible;

    LocalObject->klass = OrgObject->klass;

    giram_object_build_triangle_mesh(LocalObject);
    return LocalObject;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyTexture
******************************************************************************/
TextureStruct *CopyTexture(TextureStruct *OrgTexture)
{
  TextureStruct *LocalTexture;

  if (OrgTexture)
  {
    LocalTexture = g_new(TextureStruct, 1);
    LocalTexture->Pigment = CopyPigment(OrgTexture->Pigment);
    LocalTexture->Trans = CopyTransform(OrgTexture->Trans);
    return LocalTexture;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyPigment
******************************************************************************/
PigmentStruct *CopyPigment(PigmentStruct *OrgPigment)
{
  PigmentStruct *LocalPigment;

  if (OrgPigment)
  {
    LocalPigment = g_new(PigmentStruct, 1);
    LocalPigment->Type = OrgPigment->Type;
    LocalPigment->Color[0] = OrgPigment->Color[0];
    LocalPigment->Color[1] = OrgPigment->Color[1];
    LocalPigment->Color[2] = OrgPigment->Color[2];
    LocalPigment->Color[3] = OrgPigment->Color[3];
    LocalPigment->Color[4] = OrgPigment->Color[4];
    LocalPigment->Color2[0] = OrgPigment->Color2[0];
    LocalPigment->Color2[1] = OrgPigment->Color2[1];
    LocalPigment->Color2[2] = OrgPigment->Color2[2];
    LocalPigment->Color2[3] = OrgPigment->Color2[3];
    LocalPigment->Color2[4] = OrgPigment->Color2[4];
    LocalPigment->Color3[0] = OrgPigment->Color3[0];
    LocalPigment->Color3[1] = OrgPigment->Color3[1];
    LocalPigment->Color3[2] = OrgPigment->Color3[2];
    LocalPigment->Color3[3] = OrgPigment->Color3[3];
    LocalPigment->Color3[4] = OrgPigment->Color3[4];
    LocalPigment->Trans = CopyTransform(OrgPigment->Trans);
    return LocalPigment;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyCamera
******************************************************************************/
CameraStruct *CopyCamera(CameraStruct *OrgCamera)
{
  CameraStruct *LocalCamera;

  if (OrgCamera)
  {
    LocalCamera = g_new(CameraStruct, 1);
    LocalCamera->Type = OrgCamera->Type;
    LocalCamera->CylinderType = OrgCamera->CylinderType;
    Vcopy(LocalCamera->Location,  OrgCamera->Location);
    Vcopy(LocalCamera->LookAt,    OrgCamera->LookAt);
    Vcopy(LocalCamera->Right,     OrgCamera->Right);
    Vcopy(LocalCamera->Up,        OrgCamera->Up);
    Vcopy(LocalCamera->Direction, OrgCamera->Direction);
    Vcopy(LocalCamera->Sky,       OrgCamera->Sky);
    LocalCamera->Angle = OrgCamera->Angle;
    LocalCamera->BlurSamples = OrgCamera->BlurSamples;
    LocalCamera->Aperture = OrgCamera->Aperture;
    Vcopy(LocalCamera->FocalPoint, OrgCamera->FocalPoint);
    LocalCamera->Confidence = OrgCamera->Confidence;
    LocalCamera->Variance = OrgCamera->Variance;
    return LocalCamera;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyLightSource
******************************************************************************/
LightSourceStruct *CopyLightSource(LightSourceStruct *OrgLightSource)
{
  LightSourceStruct *LocalLightSource;

  if (OrgLightSource)
  {
    LocalLightSource = g_new(LightSourceStruct, 1);
    Vcopy(LocalLightSource->Location, OrgLightSource->Location);
    LocalLightSource->Color[0] = OrgLightSource->Color[0];
    LocalLightSource->Color[1] = OrgLightSource->Color[1];
    LocalLightSource->Color[2] = OrgLightSource->Color[2];
    LocalLightSource->Color[3] = OrgLightSource->Color[3];
    LocalLightSource->Color[4] = OrgLightSource->Color[4];
    LocalLightSource->Type = OrgLightSource->Type;
    Vcopy(LocalLightSource->PointAt, OrgLightSource->PointAt);
    LocalLightSource->Radius = OrgLightSource->Radius;
    LocalLightSource->FallOff = OrgLightSource->FallOff;
    LocalLightSource->Tightness = OrgLightSource->Tightness;
    LocalLightSource->AreaLight = OrgLightSource->AreaLight;
    Vcopy(LocalLightSource->Area1, OrgLightSource->Area1);
    Vcopy(LocalLightSource->Area2, OrgLightSource->Area2);
    LocalLightSource->AreaSize1 = OrgLightSource->AreaSize1;
    LocalLightSource->AreaSize2 = OrgLightSource->AreaSize2;
    LocalLightSource->Adaptive = OrgLightSource->Adaptive;
    LocalLightSource->Jitter = OrgLightSource->Jitter;
    LocalLightSource->LooksLike = CopyObject(OrgLightSource->LooksLike);
    LocalLightSource->FadeDistance = OrgLightSource->FadeDistance;
    LocalLightSource->FadePower = OrgLightSource->FadePower;
    LocalLightSource->AtmosphericAttenuation = OrgLightSource->AtmosphericAttenuation;
    LocalLightSource->Atmosphere = LocalLightSource->Atmosphere;
    return LocalLightSource;
  } else
    return NULL;
}

/*****************************************************************************
*  CopyTransform
******************************************************************************/
TransformStruct *CopyTransform(TransformStruct *OrgTransform)
{
  TransformStruct *LocalTransform;
  int i,j;

  if (OrgTransform)
  {
    LocalTransform = g_new(TransformStruct, 1);
    for (i=0 ; i<4 ; i++)
      for (j=0 ; j<4 ; j++)
      {
        LocalTransform->Direct[i][j] = OrgTransform->Direct[i][j];
        LocalTransform->Inverse[i][j] = OrgTransform->Inverse[i][j];
      }
    return LocalTransform;
  } else
    return NULL;
}

